#!/bin/bash

###############################################################################
#
#   This file is provided under a dual BSD/GPLv2 license.  When using or
#   redistributing this file, you may do so under either license.
#
#   GPL LICENSE SUMMARY
#
#   Copyright(c) 2007-2013 Intel Corporation. All rights reserved.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of version 2 of the GNU General Public License as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
#   The full GNU General Public License is included in this distribution
#   in the file called LICENSE.GPL.
#
#   Contact Information:
#   Intel Corporation
#
#   BSD LICENSE
#
#   Copyright(c) 2007-2013 Intel Corporation. All rights reserved.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Intel Corporation nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
###############################################################################

#####################################################################
# Name         : install.sh
#
# Purpose      : To install QuickAssist enabled ZLIB
#
# Cmdline Arg  : [Optional] -BUILD_PATH <PATH> : To build zlib directory
#                [Optional] All Config parameter that has to be passed to zlib configure script.
#
# Env Var      : ICP_ROOT/ICP_BUILD_OUTPUT should be set
#
# Info         : zlib-1.2.8.tar.gz and compressed patch file has to be placed in the 
#                same directory where this script is present.
#####################################################################
ZLIB_FILE="zlib-1.2.8.tar.gz"
CUR_DIR=$PWD
LOG_FILE="$CUR_DIR/install.log"
MEM_PATH="/contrib/qat/qat_mem/"
ZLIB_PATH="/zlib-1.2.8";
QAT_PATCH_NAME="zlib-1.2.8-qat.patch"
#COMMON_MEM_DIR="$ICP_ROOT/quickassist/utilities/libqae_mem/"
COMMON_MEM_DIR="UNKNOWN"
DH895XCC=1
DH89XXCC=2
MUX=3
BUILD=1
INSTALL=2
UNINSTALL=3

BUILD_PATH="";
async_patch_name=""
qat_patch_name=""
shim_version=""
boardType=""
zlib_config_param=""
numOfdh895=""
numOfdh89x=""
operation=

####################################################################
# Function Name       : echoMsg
# Purpose             : To print message and log in a file
# Input               : Message
# Output              : None
####################################################################
function echoMsg
{
    date +"%d-%m-%Y %T: $1" |tee -a $LOG_FILE
}

####################################################################
# Function Name       : errorExit
# Purpose             : To print message and exit
# Input               : Error Message
# Output              : None
####################################################################
function errorExit
{
    echoMsg ""
    echoMsg "ERROR: $1"
    echoMsg "Exiting........."
    echoMsg ""
    exit
}

####################################################################
# Function Name       : executeCommand
# Purpose             : To execute command passed and exit on error
# Input               : Command and Error Message
# Output              : None
####################################################################
function executeCommand
{
    cmd=$1
    errMsg=$2
    echoMsg "INFO: Command to be executed: $cmd"
    $cmd
    ret=$?
    echoMsg "INFO: Output: $ret"
    if [ "$ret" != "0" ]
    then
        errorExit "$errMsg"
    fi
}

####################################################################
# Function Name       : checkEnv
# Purpose             : To check the environment variable is set 
# Input               : None
# Output              : None
####################################################################
function checkEnv
{
    echoMsg "INFO: Checking the environment Variable"
    if [ -z "$ICP_ROOT" ] 
    then
        errorExit "ICP_ROOT is not set. ICP_ROOT should be set to QuickAssist Driver Package"
    elif [ ! -d "$ICP_ROOT" ]
    then
        errorExit "$ICP_ROOT[ICP_ROOT] doesn't exists "
    fi
    
    if [ -z "$ICP_BUILD_OUTPUT" ] 
    then
        errorExit "ICP_BUILD_OUTPUT is not set. ICP_BUILD_OUTPUT should be set to QuickAssist Driver build output directory"
    elif [ ! -d "$ICP_BUILD_OUTPUT" ]
    then
        errorExit "$ICP_BUILD_OUTPUT[ICP_BUILD_OUTPUT] doesn't exists "
    fi    

    if [ $operation -eq $UNINSTALL ]
    then
        if [ -z $ZLIB_ROOT ]
        then
            errorExit "Set ZLIB_ROOT and try again"
        elif [ ! -d $ZLIB_ROOT ]
        then
            errorExit "ZLIB_ROOT=$ZLIB_ROOT doesn't exist"
        fi
        return
    fi
     
    if [ -z "$BUILD_PATH" ] 
    then
        echoMsg "INFO: -BUILD_PATH not provided in command line argument"
        echo -e "\n\n\t======================================================="
        echo -e "\tWhere Would you like to Build the ZLIB with QAT?"
        echo -en "\tEnter to Accept Default [$CUR_DIR]:"
        read BUILD_PATH
       
        if [ -z "$BUILD_PATH" ]
        then
            BUILD_PATH=$CUR_DIR
        elif [ ! -d $BUILD_PATH ]
        then
            echoMsg "ERROR: Path provide [$BUILD_PATH] doesn't exists"
            echoMsg "Exiting.........."
            exit
        fi
        ZLIB_PATH="$BUILD_PATH/$ZLIB_PATH"
    fi
    echoMsg "INFO: Finished checking the environment Variable"
}


####################################################################
# Function Name       : detectAcceleratorTypes
# Purpose             : To detect the type of Acceleration device.
# Input               : None
# Output              : None
####################################################################
function detectAcceleratorTypes
{
    echoMsg "INFO: Detecting the Type of Acceleration device"

    numOfdh895=`lspci |grep -c 0435`
    numOfdh89x=`lspci |grep -c 0434`
    mux_status=`lsmod |grep -c qat_mux`
    
    if [ $numOfdh895 -gt 0 -a $numOfdh89x -gt 0 -a $mux_status -gt 0 ]
    then
        echoMsg "INFO: Both QAT1_5 & QAT1_6 devices detected : MUX is enabled"
        echoMsg "INFO: Building SHIMS with MUX support"
        boardType=$MUX 
    elif [ $numOfdh895 -gt 0 -a $numOfdh89x -gt 0 ]
    then
        echoMsg "INFO: Both QAT1_5 & QAT1_6 devices detected"
        while true
        do
            echo -e "\n\t QAT1_5 & QAT1_6 devices detected"
            echo -e "\t ==================================="
            echo -ne "\t Would you like to install with MUX support [YES/NO]: "
            read option
            if [ "$option" != "YES" ] && [ "$option" != "NO" ]
            then
                echo -e "\t Invalid option!!!! Try Again" 
            else
                break
            fi
        done
     
        if [ "$option" == "YES" ]
        then
            echoMsg "INFO: Building SHIMS with MUX support"
            boardType=$MUX 
        else
            echoMsg "INFO: Building SHIMS without MUX support"
            while true
            do
                echo -e "\n\t Please Select Acceleration Device Option:"
                echo -e "\t =============================="  
                echo -e "\t1. Build QuickAssist enabled zlib for DH895XCC"
                echo -e "\t2. Build QuickAssist enabled zlib for DH89XXCC"
                read option
                if [ "$option" != "1" ] && [ "$option" != "2" ]
                then
                    echo -e "\t Invalid option!!!! Try Again" 
                else
                    break
                fi
            done
            if [ "$option" == "1" ]
            then
                echoMsg "INFO: Type of Acceleration device selected is DH895XCC"
                boardType=$DH895XCC
                export ZLIB_DH895XCC=1
            else 
                echoMsg "INFO: Type of Acceleration device selected is DH89XXCC"
                boardType=$DH89XXCC
            fi
        fi
    elif [ $numOfdh895 -gt 0 ]
    then
        echoMsg "INFO: Type of Acceleration device is DH895XCC"
        boardType=$DH895XCC
        export ZLIB_DH895XCC=1
    elif [ $numOfdh89x -gt 0 ]
    then
        echoMsg "INFO: Type of Acceleration device is DH89XXCC"
        boardType=$DH89XXCC
    fi

    if [ -z "$boardType" ]
    then
        echoMsg "INFO: No Acceleration device available"
        disabled="[DISABLED]"
        while [ $loopB -eq 0 ];
        do
            echo -e "\n\n\t Please Select Acceleration Device Option:"
            echo -e "\t ========================================="    
            echo -e "\t1. Build QuickAssist enabled zlib for DH895XCC"
            echo -e "\t2. Build QuickAssist enabled zlib for DH89XXCC"
            echo -en "\tOption:"
            read optDev
            loopB=1
            case $optDev in

                1)
                    echoMsg "INFO: Selected Acceleration device DH895XCC"
                    boardType=$DH895XCC
                    export ZLIB_DH895XCC=1
                ;;

                2)
                    echoMsg "INFO: Selected Acceleration device DH89XXCC"
                    boardType=$DH89XXCC
                ;;

                *)
                    echo -e "Invalid Option Entered. Please enter valid option"
                    loopB=0
                ;;
            esac
        done     
    fi

    if [ -z $boardType ]
    then
        errorExit "Failed to detect the Acceleration device"
    fi
    if [ $boardType == $MUX ]
    then
        export WITH_CPA_MUX=1
    fi
    echoMsg "INFO: Detected the type of Acceleration device"
}

####################################################################
# Function Name       : backupZlib
# Purpose             : To backup zlib directory if exists.
# Input               : None
# Output              : None
####################################################################
function backupZlib
{
    echoMsg "INFO: Taking backup of existing zlib directory if it exists"
    #backup ZLIB directory if already present.
    if [ -d $ZLIB_PATH ]
    then
        echoMsg "INFO: zlib directory exists"
        curTime=`date +"%N"`
        mv $ZLIB_PATH "$ZLIB_PATH$curTime"
        echoMsg "INFO: Existing zlib directory renamed to $ZLIB_PATH$curTime"
    fi
    
}

####################################################################
# Function Name       : extractZlib
# Purpose             : To extract zlib in BUILD_PATH
# Input               : None
# Output              : None
####################################################################
function extractZlib
{
    echoMsg "INFO: Extracting the zlib package"
    cd $CUR_DIR
    if [ ! -e $ZLIB_FILE ]
    then
        errorExit "$ZLIB_FILE not found in the current directory $CUR_DIR"
    fi
    
    cd $BUILD_PATH
    executeCommand "tar zxof $CUR_DIR/$ZLIB_FILE" "Failed to extract $CUR_DIR/$ZLIB_FILE";
    echoMsg "INFO: Extracted $ZLIB_FILE in $BUILD_PATH successfully" 
   
    cd $CUR_DIR
    echoMsg "INFO: Completed extracting the zlib package"
}

####################################################################
# Function Name       : extractPatch
# Purpose             : Extract qat patch present in 
#                       current directory
# Input               : None
# Output              : None
####################################################################
function extractPatch
{
    echoMsg "INFO: Extracting the patch"
    cd $CUR_DIR
    num=`ls |egrep -c 'zlib-1.2.8-qat.L.*.tar.gz'`

    #If Multiple files present inside the directory. Request operator to select the version.
    if [ $num -gt 1 ]
    then
        echoMsg "INFO: Multiple patch files found in the directory"
        index=1;
        verArray=(0)

        echo -e "\n\n\t================================================="
        echo -e "\tMultiple versions present inside the directory."
        echo -e "\tPlease select the version to be installed:"

        for file in `ls zlib-1.2.8-qat.L.*.tar.gz`
        do
            version=`echo $file|sed 's/zlib-1.2.8-qat.L.//g;s/.tar.gz//g'` 
            echo -e "\t$index: $version"
            verArray[$index]=$version
            index=$((index+1))
        done

        if [ $index -eq 1 ]
        then
            errorExit "ZLIB Qat Patch not found"
        fi
        
        echo -ne "\tPlease enter the index:"
        read versIndex
        if [ -z $versIndex ] || [ $versIndex -gt $index ] || [ "$versIndex" == "0" ]
        then
            errorExit "Invalid Option provided"
        fi
        shim_version=${verArray[$versIndex]}
        
    elif [ "$num" -eq 1 ]
    then
        echoMsg "INFO: Only one version of patch files found in the directory"
        shim_version=`ls zlib-1.2.8-qat.L.*.tar.gz|sed 's/zlib-1.2.8-qat.L.//g;s/.tar.gz//g'`
        echoMsg "INFO: QuickAssist enabled ZLIB Version: $shim_version"
    else
        errorExit "Patch files not found in $CUR_DIR"
    fi

    #Extract the package of selected version
    cd $ZLIB_PATH/../
    executeCommand "tar zxof $CUR_DIR/zlib-1.2.8-qat.L.$shim_version.tar.gz" "Failed to extract qat Patch $CUR_DIR/zlib-1.2.8-qat.L.$shim_version.tar.gz"
    
    cd $CUR_DIR
    echoMsg "INFO: Finished extracting the Patch files"
}

####################################################################
# Function Name       : applyPatch
# Purpose             : To apply the  Patch
# Input               : Patch File Name
# Output              : None
####################################################################
function applyPatch
{
    patchName=$1
    echoMsg "INFO: Applying the Patch file $patchName"
    cd $ZLIB_PATH
    patch -p0 < ../$patchName
    if [ $? != "0" ]
    then
        errorExit "Failed to apply Patch $patchName"
    fi

    cd $CUR_DIR
    echoMsg "INFO: Completed applying the Patch file $patchName"
}

####################################################################
# Function Name       : buildZLib
# Purpose             : To compile and build zlib
# Input               : None
# Output              : None
####################################################################
function buildZlib
{
    echoMsg "INFO: Building the Zlib with patch file applied"
    cd $ZLIB_PATH
    export ZLIB_ROOT=$ZLIB_PATH

    echoMsg "INFO: Configuring zlib with options $zlib_config_param"
    executeCommand "./configure $zlib_config_param" "zlib configure command failed"

    echoMsg "INFO: Executing make "
    executeCommand "make" "make failed"

    echoMsg "INFO: make zpipe"
    executeCommand "make zpipe" "make failed"

    echoMsg "INFO: make comptestapp"
    cd $ZLIB_PATH/contrib/qat/qat_zlib_test/
    executeCommand "make" "make failed"

    cd $CUR_DIR

    echoMsg "INFO: Finished building the Zlib with patch file applied"
}

####################################################################
# Function Name       : buildMemoryDriver
# Purpose             : To build Memory Driver
# Input               : None
# Output              : None
####################################################################
function buildMemoryDriver
{
    if [ -d $COMMON_MEM_DIR ]
    then
        echoMsg "INFO: Common Memory driver detected. Building common memory driver"
        cd $COMMON_MEM_DIR
        source ./setenv.sh
        KERNEL_PATH="/usr/src/kernels/`\uname -r`"
        if [ `uname -r|grep -c grsec-WR` -gt 1 ]
        then
            KERNEL_PATH="/lib/modules/`\uname -r`"
        fi
        export KERNEL_SOURCE_ROOT=$KERNEL_PATH

        executeCommand "make" "make failed"
        echoMsg "INFO: Finished building the Common Memory driver"
    else
        echoMsg "INFO: Building the QAT Memory driver"
        cd $ZLIB_PATH/$MEM_PATH
        executeCommand "make" "Make failed for memory driver"
        echoMsg "INFO: Finished building the QAT Memory driver"
    fi
    cd $CUR_DIR
}

####################################################################
# Function Name       : installShim
# Purpose             : To inject and start the driver
# Input               : None
# Output              : None
####################################################################
function installShim
{
    echoMsg "INFO: Installing QuickAssist enabled zlib"

    #Check Driver is already injected
    qatModStatus=`lsmod|grep -c icp_qa_al`;
    if [ $qatModStatus -gt 0 ]
    then
        status=`$ICP_BUILD_OUTPUT/adf_ctl status|grep -c up`
        if [ $status -gt 0 ]
        then
            #Stop the QAT Service
            echoMsg "INFO: Bringing down Driver"
            $ICP_BUILD_OUTPUT/adf_ctl down
        fi
    fi
    
    #Take Backup of the config file
    curTime=`date +"%N"`
    echoMsg "INFO: Backingup current config files into /etc/default_conf_file_$curTime"
    mkdir -p /etc/default_conf_file_$curTime
    
    if [ $boardType == $DH895XCC -o $boardType == $MUX ]
    then
        mv /etc/dh895xcc_qa_dev?.conf /etc/default_conf_file_$curTime
        for (( i=0; i<$numOfdh895; i++ ))
        do
            cp $ZLIB_PATH/contrib/qat/config/dh895xcc/multi_process_optimized/dh895xcc_qa_dev0.conf /etc/dh895xcc_qa_dev$i.conf
        done
    fi
    if [ $boardType == $DH89XXCC -o $boardType == $MUX ]
    then
        mv /etc/dh89xxcc_qa_dev?.conf /etc/default_conf_file_$curTime
        for (( i=0; i<$numOfdh89x; i++ ))
        do
            cp $ZLIB_PATH/contrib/qat/config/dh89xxcc/multi_process_optimized/dh89xxcc_qa_dev0.conf /etc/dh89xxcc_qa_dev$i.conf
        done
    fi

    
    #Eject QAT_MEM Memory Driver if loaded
    if [ `lsmod|grep -c qat_mem` -gt 0 ]
    then
        echoMsg "INFO: Removing qat_mem driver module"
        rmmod qat_mem
    fi
    
    #Eject COMMON Memory Driver if loaded 
    if [ `lsmod|grep -c libqae_mem` -gt 0 ]
    then
        echoMsg "INFO: Removing libqae_mem driver module"
        rmmod libqae_mem
    fi
    
    #Insert Memory Driver
    if [ -d "$COMMON_MEM_DIR" ]
    then
        echoMsg "INFO: Inserting the Common Memory Module"
        cd $COMMON_MEM_DIR
        executeCommand "insmod ./libqae_mem.ko" "Failed to insert common memory Driver"
    else
        echoMsg "INFO: Inserting the QAT Memory Module"
        cd $ZLIB_PATH/$MEM_PATH
        executeCommand "insmod ./qat_mem.ko" "Failed to insert memory Driver"
    fi

    if [ $boardType != $MUX ]
    then
        if [ $qatModStatus -eq "0" ]
        then
            echoMsg "INFO: Injecting the QAT Driver Module"
            executeCommand "insmod $ICP_BUILD_OUTPUT/icp_qa_al.ko" "Failed to insert module $ICP_BUILD_OUTPUT/icp_qa_al.ko"
        fi

        #Start the driver
        echoMsg "INFO: Bringing up the QAT Driver"
        ret=`$ICP_BUILD_OUTPUT/adf_ctl up|grep -ci error`
        if [ $ret -gt 0 ]
        then
            errorExit "Failed to bring up the driver"
        fi
    else
        if [ `/etc/init.d/qat_service stop|egrep -ci "FATAL|ERROR"` -gt 0 ]
        then
            echoMsg "ERROR: Failed to stop the driver."
        fi
        if [ `/etc/init.d/qat_service start|egrep -ci "FATAL|ERROR"` -gt 0 ]
        then
            errorExit "Failed to bring up the driver with MUX support"
        fi
    fi

    echoMsg "INFO: Finished installing and bringing up the driver with new configuration"
}

####################################################################
# Function Name       : verifyBuild
# Purpose             : To verify the status of zlib build
# Input               : None
# Output              : None
####################################################################
function verifyBuild
{
    echoMsg "INFO: Verify the Build status"
    cd $ZLIB_PATH
    ls -l > file.txt
    executeCommand "./minigzip file.txt" "Minigzip Test Run Failed!!!!!!!!!"
    if [ ! -e file.txt.gz ] 
    then
        errorExit "minigzip Test Run Failed!!!!!!!!!!"
    fi
    rm -rf file.txt file.txt.gz

    ls -l > file.txt
    ./zpipe < file.txt > file.txt.gz
    if [ $? -ne 0 ]
    then
        errorExit "Zpipe Test Run Failed!!!!!!!!!!!"
    fi

    ./zpipe -d < file.txt.gz > file.txt.res
    if [ $? -ne 0 ]
    then
        errorExit "Zpipe Test Run Failed!!!!!!!!!!!"
    fi
    #executeCommand "./zpipe < file.txt > file.txt.gz" "zpipe Test Run Failed!!!!!!!!!"
    #executeCommand "./zpipe -d < file.txt.gz > file.txt.res" "zpipe Test Run Failed!!!!!!!!!"

    result=`diff --brief -s file.txt file.txt.res |grep -c identical`
    if [ $result -eq 0 ] 
    then
        errorExit "zpipe Test Run Failed!!!!!!!!"
    fi
    rm -rf file.txt file.txt.res file.txt.gz
    echoMsg "INFO: Verified the Build status"
}

####################################################################
# Function Name       : unInstall
# Purpose             : To remove the Zlib
# Input               : None
# Output              : None
####################################################################
function unInstall
{
    echoMsg "INFO: Uninstalling qat enabled ZLIB"

    loop2=0
    option=""
    while [ $loop2 == 0 ];
    do
        echo -e "\n\t The following folders will be deleted :"
        echo -e "\t ==================================="
        echo -e "\t 1. $ZLIB_ROOT"
        echo -e "\t WARNING: Please ensure that no other applications have dependencies to these libraries before uninstalling them"
        echo -ne "\t Would like to proceed further [YES/NO]: "
        read option
        option=`echo $option|tr [a-z] [A-Z]`

        if [ "$option" == "YES" ] || [ "$option" == "NO" ]
        then
            loop2=1
        fi
    done

    if [ "$option" == "YES" ]
    then

        #Removing qat_mem
        qatMemStatus=`lsmod|grep -c qat_mem`
        if [ $qatMemStatus -gt 0 ]
        then
            loop2=0
            while [ $loop2 == 0 ];
            do
                echo -ne "\n\n\t QAT Mem module detected"
                echo -ne "\t Would like to remove qat_mem module [YES/NO]: "
                read option
                option=`echo $option|tr [a-z] [A-Z]`

                if [ "$option" == "YES" ] || [ "$option" == "NO" ]
                then
                    loop2=1
                fi
            done 
            
            if [ "$option" == "YES" ]
            then
                echoMsg "Removing QAT Memory Module"
                rmmod qat_mem
                rm -rf /dev/qat_mem
            fi
        fi

        echoMsg "INFO: Removing the ZLIB_ROOT[$ZLIB_ROOT] directory"
        rm -rf $ZLIB_ROOT
        
        for file in `ls $ICP_BUILD_OUTPUT/*.conf.v2`
        do
            dest=`echo $file|sed 's/\.v2$//'`
            echoMsg "Copying files $file to $dest"
            cp $file $dest
        done

        #Check Driver is already injected
        qatModStatus=`lsmod|grep -c icp_qa_al`;
        if [ $qatModStatus -gt 0 ]
        then
            status=`$ICP_BUILD_OUTPUT/adf_ctl status|grep -ci up`
            if [ $status -gt 0 ]
            then
                echoMsg "INFO: Bringing down the QAT Driver"
                ret=`$ICP_BUILD_OUTPUT/adf_ctl down|grep -ci error`
                if [ $ret -gt 0 ]
                then
                    errorExit "Failed to bring down the driver"
                fi

                echoMsg "INFO: Bringing up the QAT Driver"
                ret=`$ICP_BUILD_OUTPUT/adf_ctl up|grep -ci error`
                if [ $ret -gt 0 ]
                then
                    errorExit "Failed to bring up the driver"
                fi
            fi
        fi

        echoMsg "INFO: Successfully Uninstalled the zlib"
    else
        echoMsg "INFO: Quitting uninstall"
    fi
    
}
    
    


####################################################################
# Function Name       : mainMenu
# Purpose             : To prompt user for buildOptions.
# Input               : None
# Output              : None
####################################################################
function mainMenu
{

    loop=0
    while [ $loop == 0 ];
    do
        echo -e "\n\n\t Please Select Install Option :"
        echo -e "\t ================================"
        echo -e "\t 1  Build QuickAssist enabled zlib"
        echo -e "\t 2  Install QuickAssist enabled zlib"
        echo -e "\t 3  Uninstall"
        echo -e "\t 0  Exit"
        echo -en "\t Option: "
        read case
        
        case $case in

            1 )
                echo -e "\n\t****Building QuickAssist enabled zlib****>\n\n"
                operation=$BUILD
                checkEnv
                detectAcceleratorTypes
                backupZlib
                extractZlib
                extractPatch
                applyPatch $QAT_PATCH_NAME
                buildMemoryDriver
                buildZlib
                echo -e "\n\n\n\t***QuickAssist enabled zlib Build Complete***>\n\n"
            ;;

            2 )
                echo -e "\n\t****Installing QuickAssist enabled zlib****>\n\n"
                operation=$INSTALL
                checkEnv
                detectAcceleratorTypes
                backupZlib
                extractZlib
                extractPatch
                applyPatch $QAT_PATCH_NAME
                buildMemoryDriver
                buildZlib
                installShim
                verifyBuild
                echo -e "\n\n\n\t***QuickAssist enabled zlib Install Complete***>\n\n"
            ;;

            3 )
                echo -e "\n\t****Uninstalling QuickAssist enabled zlib****>\n\n"
                operation=$UNINSTALL
                checkEnv
                unInstall 
            ;;

            0)
                echo -e "\n\t****EXITING INSTALLER****>\n\n"
                exit
            ;;

            *)
                echo -e "\n\t****Invalid Option****>"
                echo -e "\n\t****Please Choose Again****>\n\n"
            ;;
        esac
    done
}


####################################################################
# Function Name       : main
# Purpose             : Main function that invoke all the other function
#                       Starting point of execution
# Input               : None
# Output              : None
####################################################################
function main
{
    echo -e "\n\n\t==============================================="
    echo -e "\tWelcome to Intel(R) QuickAssist enabled zlib Interactive Installer"
    echo -e "\t===============================================\n\n\n\n"
    
    #Display Menu
    mainMenu
}


#Parse Command line argument.
while [ "$1" != "" ]
do

    case $1 in
        -BUILD_PATH)
            BUILD_PATH=$2
            if [ ! -d $BUILD_PATH ]
            then
                errorExit "Dir $BUILD_PATH doesn't exists"
            fi
            shift 2
            ZLIB_PATH="$BUILD_PATH/$ZLIB_PATH"
            ;;
        *)
            zlib_config_param="$zlib_config_param $1"
            shift
            ;;
    esac
done

#Call Main function to perform all the operations.
main;

